#!/bin/bash
set +x

# Get options
PROCS_PER_NODE=8
while getopts 'h:c:p:' flag; do
  case "${flag}" in
    h) HOSTS_FILE="${OPTARG}" ;;
    c) CMD_FILE="${OPTARG}" ;;
    p) PROCS_PER_NODE="${OPTARG}" ;;
    *) printf "Usage: launch.sh [-h] hosts [-c] cmd [p] procs_per_node"
       exit 1 ;;
  esac
done

# Rendezvous
RNDV_DIRNAME=$RANDOM
export RNDV_PATH="/checkpoint/jacobkahn/rndv/${RNDV_DIRNAME}"
mkdir ${RNDV_PATH}
export RNDV_FILE="${RNDV_PATH}/${RANDOM}.rndv"
touch ${RNDV_FILE}

# Read hosts
readarray -t HOSTS < ${HOSTS_FILE}
export WORLD_SIZE=$((${#HOSTS[@]} * ${PROCS_PER_NODE}))
echo "World Size: ${WORLD_SIZE}"
echo "Hosts: ${HOSTS[*]}"
export HOSTS_FILE=${HOSTS_FILE}

# Launch procs
for i in "${!HOSTS[@]}"; do
    for (( j=0; j<${PROCS_PER_NODE}; j++ ))
    do
        export HOST="${HOSTS[$i]}"
        export RANK=$((i * ${PROCS_PER_NODE} + j))
        echo "Rank ${RANK} - host ${HOST}"

        # Prepare the command with this rank and rendezvous path
        CMD=$(envsubst < ${CMD_FILE})
        
        ssh $HOST "bash --login -c '${CMD}'" &
        # ssh $HOST bash -c ". ~/.bashrc; ${CMD}" &
    done
done
