# An LM model that's a simple stack of transformers with a sinusoidal
# # positional embedding. Uses a fixed dictionary size of 200k

from transformer import TransformerDecoderLayer
from sinpos import SinusoidalPositionalEmbedding
from torch.nn import Dropout, Embedding, Module, Linear, ModuleList
import math

class LMModel(Module):
    def __init__(self, d_model, nhead, dim_feedforward=2048, dropout=0.1, activation="relu", num_tokens=100000, num_transformer_layers=16):
        super().__init__()
        self.d_model = d_model
        decoder = []
        for i in range(num_transformer_layers):
            decoder.append(TransformerDecoderLayer(
                d_model=d_model,
                nhead=nhead,
                dim_feedforward=dim_feedforward,
                dropout=dropout,
                activation=activation)
            )

        self.decoder = ModuleList(decoder)

        self.sinpos = SinusoidalPositionalEmbedding(
            embedding_dim=d_model, padding_idx=0)
        self.emb = Embedding(num_tokens + 1, d_model)
        self.dp = Dropout(dropout)
        self.linear = Linear(d_model, num_tokens + 1, bias=True)

    def forward(
        self,
        input
    ):
        x = self.emb(input)
        x = self.dp(math.sqrt(self.d_model) * x + self.sinpos(input))
        for m in self.decoder:
            x = m(x)
        x = self.linear(x) # map from tr output back into num_tokens
        return x
