import tensorflow as tf
from tensorflow.keras import layers, models, regularizers

def alexnet(img_shape, n_classes):
    print(f"Creating alexnet with input shape {img_shape} and number of classes {n_classes}")
    model = tf.keras.Sequential([
        # layer 1
        tf.keras.layers.ZeroPadding2D(2, input_shape=img_shape),
        tf.keras.layers.Conv2D(filters=64,
                               kernel_size=(11, 11),
                               strides=4,
                               padding="valid",
                               activation=tf.keras.activations.relu),
        tf.keras.layers.MaxPool2D(pool_size=(3, 3),
                                  strides=2,
                                  padding="valid"),
        # layer 2
        tf.keras.layers.ZeroPadding2D(2),
        tf.keras.layers.Conv2D(filters=192,
                               kernel_size=(5, 5),
                               strides=1,
                               padding="valid",
                               activation=tf.keras.activations.relu),
        tf.keras.layers.MaxPool2D(pool_size=(3, 3),
                                  strides=2,
                                  padding="valid"),
        # layer 3
        tf.keras.layers.ZeroPadding2D(1),
        tf.keras.layers.Conv2D(filters=384,
                               kernel_size=(3, 3),
                               strides=1,
                               padding="valid",
                               activation=tf.keras.activations.relu),
        # layer 4
        tf.keras.layers.ZeroPadding2D(1),
        tf.keras.layers.Conv2D(filters=256,
                               kernel_size=(3, 3),
                               strides=1,
                               padding="valid",
                               activation=tf.keras.activations.relu),
        # layer 5
        tf.keras.layers.ZeroPadding2D(1),
        tf.keras.layers.Conv2D(filters=256,
                               kernel_size=(3, 3),
                               strides=1,
                               padding="valid",
                               activation=tf.keras.activations.relu),
        tf.keras.layers.MaxPool2D(pool_size=(3, 3),
                                  strides=2,
                                  padding="valid"),
        # layer 6
        tf.keras.layers.Flatten(),
        tf.keras.layers.Dropout(rate=0.2),
        tf.keras.layers.Dense(units=4096,
                              activation=tf.keras.activations.relu),
        tf.keras.layers.Dropout(rate=0.2),
        # layer 7
        tf.keras.layers.Dense(units=4096,
                              activation=tf.keras.activations.relu),
        tf.keras.layers.Dropout(rate=0.2),
        # layer 8
        tf.keras.layers.Dense(units=n_classes,
                              activation=tf.keras.activations.softmax)
    ])
    
    model.build(img_shape)
    return model
